/* $Id: lists.h,v 1.41 2004/07/14 00:24:24 jonas Exp $ */

#ifndef HED__UTIL_TREE_H
#define HED__UTIL_TREE_H

/* Behold, a tree! */
struct tree {
	void *root;
	off_t root_ofs;
};

/* This must be at the start of each item. When you are declaring
 * the item structure, always use the TREE_HEAD macro. */
struct tree_head {
	/* Can be NULL for the root. */
	struct tree_head *up;
	/* Both can be NULL for leaves. */
	struct tree_head *left;
	struct tree_head *right;
	/* Size of the entry. (This is public.) */
	off_t size, phys_size;
	/* Size of the entry plus its children. (Private.) */
	off_t cover_size, cover_phys_size;
};


/* These prototypes may be in fact defined as macros below! */

/* WARNING: Be very careful when updating size (directly or e.g. by unchaining).
 * If you e.g. do update_size() && next_block() && recalc(), you're screwed.
 * Anything can splay so you MUST get them back in sync BEFORE calling ANY tree
 * routine. */

struct tree *init_tree(void);
/* Update node information */
void recalc_node(void *item);
void recalc_node_recursive(void *item);

/* Delete from the tree */
void del_from_tree(struct tree *tree, void *item);
/* Delete from the tree, but do NOT recalculate parent nodes. Use this when
 * replacing one node with another (equally sized). However, you SHOULD
 * recalculate when merging two nodes together etc. - the cover sizes
 * might not be right if you merged a child in. */
struct tree_head *unchain_from_tree(struct tree *tree, void *item);

void add_to_tree(struct tree *tree, off_t offset, void *item);
void add_after_in_tree(struct tree *tree, void *pos, void *item);


/* Finds the item at or right before the given offset (except when there is
 * no item at or before the offset; then it gives the nearest item after
 * the offset; you can distinguish the case by item->left being NULL. */
void *find_in_tree(struct tree *tree, off_t offset);


void *first_in_tree(struct tree *tree);
void *prev_in_tree(struct tree *tree, void *item);
void *next_in_tree(struct tree *tree, void *item);
void *last_in_tree(struct tree *tree);
off_t tree_block_offset(void *block);
off_t tree_block_phys_offset(void *block);

/* You shouldn't need to call these directly unless you want to
 * do special optimizations for certain items. */
void splay(struct tree *tree, void *item);
void unsplay(struct tree *tree, void *item);

#define foreach_in_tree(e, t) \
	for ((e) = first_in_tree(t); (e) != NULL; (e) = next_in_tree((t), (e)))

#define foreachsafe_in_tree(e, n, t) \
	for ((e) = first_in_tree(t), (n) = next_in_tree(t, (e)); \
	     (e) != NULL; (e) = (n), (n) = next_in_tree((t), (e)))

#endif
